/*\file vec2.h
*/
/*!    vec2 is a two dimensional point
*/

#pragma once
#include <iostream>
#include <assert.h>
#include <math.h>


//! defines two-dimensional point
class vec2 {
public:	
	vec2() { vec2(0.0,0.0); } //!< default no-argument constructor
	~vec2() {} //!< destructor
	//! constructs vec2 from (x,y) coordinates
	vec2(double x, double y) { body[0] = x; body[1] = y; }
	//! constucts unit vector at angle theta (in degrees) from x-axis
	vec2(double theta);
	//! returns length of associated array
	int length() const { return 2; }
	//! return pointer to first element
	double * vptr() { return body; }
	//! subscript operator (non-const object)
	double &operator[](int k)  { return body[k]; }
	//! subscript operator (const object)
	const double &operator[](int k) const { return body[k]; }
	vec2 &operator+=(const vec2&b);
	//! set vec2 to new value
	vec2 &set(double x, double y) { body[0]=x; body[1]=y; return *this; }
	vec2 &scale(double scl) { body[0]*=scl; body[1] *=scl; return *this; }
	vec2 &rotate(double theta);
	//! translate vec2
	vec2 &translate(vec2 &v) { body[0] += v[0]; body[1] += v[1]; return *this; }
	friend void sum(vec2 &result,const vec2 &a,const vec2 &b);
	friend void diff(vec2 &result, const vec2 &a, const vec2 &b);
	double getSize() const;
	vec2 &setSize(double sz);
	//operator overloads
	friend vec2 operator+(const vec2&a, const vec2&b);
	friend vec2 operator-(const vec2&a, const vec2&b);
	friend vec2 operator-(const vec2&a);
	friend vec2 operator+(const vec2&a, double b);
	friend vec2 operator-(const vec2&a, double b);
	friend vec2 operator*(const vec2&a, double b);
	friend vec2 operator/(const vec2&a, double b);
	
	//! iostream output
	friend std::ostream& operator << (std::ostream& s, const vec2& v);
private:
	double body[2]; //!< contains the elements of a 2D point or vector
};

//! returns a += b;
inline vec2 & vec2::operator+=(const vec2 &b)
{
	body[0] += b[0];
	body[1] += b[1];
	return *this;
}


//! returns (a+b)
inline vec2 operator+(const vec2 &a, const vec2 &b)
{
	return vec2(a[0]+b[0],a[1]+b[1]);
}

inline void sum(vec2 &result, const vec2 &a, const vec2 &b)
{
	result[0] = a[0]+b[0];
	result[1] = a[1]+b[1];
}

//! returns (a-b)
inline vec2 operator-(const vec2 &a, const vec2 &b)
{
	return vec2(a[0]-b[0],a[1]-b[1]);
}

inline void diff(vec2 &result, const vec2 &a, const vec2 &b)
{
	result[0] = a[0]-b[0];
	result[1] = a[1]-b[1];
}

//! returns -a
inline vec2 operator-(const vec2 &a) { return vec2(-a[0],-a[1]); }

//! returns a + scalar
inline vec2 operator+(const vec2&a, double b) {return vec2(a[0]+b,a[1]+b); }
//! returns a - scalar
inline vec2 operator-(const vec2&a, double b) {return vec2(a[0]-b,a[1]-b); }
//! returns a * scalar
inline vec2 operator*(const vec2&a, double b) {return vec2(a[0]*b,a[1]*b); }
//! returns a / scalar
inline vec2 operator/(const vec2&a, double b) {return vec2(a[0]/b,a[1]/b); }

//! return dot product 
inline double dot(const vec2 &a, const vec2 &b)
{
	return a[0]*b[0]+a[1]*b[1];
}

//! return 2D point perpendicular to argument
inline vec2 perp(const vec2 &v) { return vec2(-v[1],v[0]); }

//! return 2-norm (vector length, distance from origin)
inline double magn(const vec2 &a)
{
	return a.getSize();
}

//! return 2-norm (vector magnitude or length, distance from origin)
inline double vec2::getSize() const
{
	const vec2 &p = *this;
	return sqrt(p[0]*p[0]+p[1]*p[1]);
}

//! set vector to specified length or magnitude
inline vec2 &vec2::setSize(double sz)
{
	vec2 &p = *this;
	double vn = getSize();
	assert(vn>0.0);
	sz /= vn;
	p[0] *= sz;
	p[1] *= sz;
	return *this;
}
	

//! return distance between two points
inline double distance(const vec2 &a, const vec2 &b)
{
	return _hypot(a[0]-b[0],a[1]-b[1]);
}


double perpdot(const vec2 &a, const vec2 &b);
double angle(const vec2 &a,const vec2 &b);
vec2 rotate(const vec2 &a, double theta);